# alias.tcl --
#
# Code which handles aliases.
#
#  TkRat software and its included text is Copyright 1996 by Martin Forssen.
#
#  The full text of our legal notices is contained in the file called
#  COPYRIGHT, included with this distribution.

# Counter used to generate uniqe window names
set aliasIdent 0

# AliasCreate --
#
# Create a new alias and insert it into the alias list.
#
# Arguments:

proc AliasCreate {} {
    global aliasIdent t option

    # Create identifier
    set id al[incr aliasIdent]
    set w .$id
    upvar #0 $id hd
    set hd(done) 0
    set hd(oldFocus) [focus]

    # Create toplevel
    toplevel $w
    wm title $w $t(alias_create)

    frame $w.name
    label $w.name.label -text $t(alias): -width 12 -anchor e
    entry $w.name.entry -textvariable ${id}(name) -width 40
    pack $w.name.label $w.name.entry -side left
    frame $w.fname
    label $w.fname.label -text $t(fullname): -width 12 -anchor e
    entry $w.fname.entry -textvariable ${id}(fname) -width 40
    pack $w.fname.label $w.fname.entry -side left
    frame $w.cont
    label $w.cont.label -text $t(alias_content): -width 12 -anchor e
    text $w.cont.text -relief sunken -width 40 -wrap word -height 5
    bindtags $w.cont.text [list Text $w.cont.text $w all]
    bind $w.cont.text <Return> {break}
    pack $w.cont.label $w.cont.text -side left -anchor n
    OkButtons $w $t(create) $t(cancel) "AliasCreate2 $w $id"

    pack $w.name \
	 $w.fname \
	 $w.cont -side top
    pack $w.buttons -pady 5 -fill x
    
    Place $w aliasCreate
    focus $w.name.entry
}
proc AliasCreate2 {w id done} {
    global option
    upvar #0 $id hd

    if $done {
	set content {}
	foreach item [split [$w.cont.text get 0.0 end] ", \t\n"] {
	    if ![regexp {^[ 	]*$} $item] {
		set content "$content,$item"
	    }
	}
	set content [string trimleft $content ,]
	if { ![string length $hd(name)] || ![string length $content] } {
	    Popup $t(need_name_and_content)
	    continue
	}
	RatAlias add $hd(name) $hd(fname) $content
	RatAlias save $option(aliases_file)
    }

    RecordPos $w aliasCreate
    focus $hd(oldFocus)
    destroy $w
    unset hd
}


# AliasList --
#
# List defined aliases.
#
# Arguments:

proc AliasList {} {
    global aliasIdent t defaultFont

    # Create identifier
    set id al[incr aliasIdent]
    set w .$id
    set oldFocus [focus]

    # Create toplevel
    toplevel $w
    wm title $w $t(alias_list)

    frame $w.f
    scrollbar $w.f.scroll -relief raised -bd 1 -highlightthickness 0 \
	    -command "$w.f.list yview"
    listbox $w.f.list -yscroll "$w.f.scroll set" -relief raised -bd 1 \
	    -exportselection false -highlightthickness 0 -font $defaultFont \
	    -selectmode single
    Size $w.f.list aliasList
    pack $w.f.scroll -side right -fill y
    pack $w.f.list -expand 1 -fill both
    button $w.create -text $t(create)... \
	    -command "AliasCreate ; AliasPopulate $w.f.list"
    button $w.edit -text $t(edit)... -command "AliasEdit $w.f.list"
    button $w.delete -text $t(delete) -command "AliasDelete $w.f.list"
    button $w.dismiss -text $t(dismiss) -command "RecordPos $w aliasList; \
	      RecordSize $w.f.list aliasList; destroy $w"

    pack $w.f -side top -expand 1
    pack $w.create \
	 $w.edit \
	 $w.delete \
	 $w.dismiss -side left -expand 1


    AliasPopulate $w.f.list

    Place $w aliasList
}


# AliasPopuplate
#
# Populate the alias list
#
# Arguments:
# l -	Listbox to populate

proc AliasPopulate l {
    $l delete 0 end
    RatAlias list alias
    foreach a [lsort [array names alias]] {
	$l insert end [format "%-10s %-20s %s" $a [lindex $alias($a) 0] \
		[lindex $alias($a) 1]]
    }
}

# AliasRead --
#
# Read aliases from default file.
#
# Arguments:

proc AliasRead {} {
    global option

    if [file exists $option(aliases_file)] {
	RatAlias read $option(aliases_file)
    }
}


# AliasDelete --
#
# Delete selected aliases from list
#
# Arguments:
# l -	Listbox from which to remove selected aliases

proc AliasDelete {l} {
    global option
    foreach index [lsort -integer -decreasing [$l curselection]] {
	RatAlias delete [lindex [$l get $index] 0]
	$l delete $index
    }
    RatAlias save $option(aliases_file)
}


# AliasEdit --
#
# Edit the selected aliases
#
# Arguments:
# l -	Listbox which contains the alias to be edited

proc AliasEdit l {
    global aliasIdent t option

    # Create identifier
    set id al[incr aliasIdent]
    set w .$id
    upvar #0 $id hd
    set hd(done) 0
    set oldFocus [focus]

    # Create toplevel
    toplevel $w
    wm title $w $t(alias_edit)

    # Populate it
    frame $w.name
    label $w.name.label -text $t(name): -width 12 -anchor e
    entry $w.name.entry -textvariable ${id}(name) -width 40
    pack $w.name.label $w.name.entry -side left
    frame $w.fname
    label $w.fname.label -text $t(fullname): -width 12 -anchor e
    entry $w.fname.entry -textvariable ${id}(fname) -width 40
    pack $w.fname.label $w.fname.entry -side left
    frame $w.cont
    label $w.cont.label -text $t(content): -width 12 -anchor e
    text $w.cont.text -relief sunken -width 40 -wrap word -height 5
    bindtags $w.cont.text [list Text $w.cont.text $w all]
    bind $w.cont.text <Return> {break}
    pack $w.cont.label $w.cont.text -side left -anchor n
    OkButtons $w $t(ok) $t(cancel) "AliasEdit2 $w $l $id"

    pack $w.name \
	 $w.fname \
	 $w.cont -side top
    pack $w.buttons -pady 5 -fill x
    
    Place $w aliasEdit
    focus $w.name.entry

    RatAlias list alias
    set alist [lsort [array names alias]]
    set index [lindex [$l curselection] 0]

    set hd(name) [lindex $alist $index]
    set hd(fname) [lindex $alias($hd(name)) 0]
    $w.cont.text delete 0.0 end
    foreach c [split [lindex $alias($hd(name)) 1] ,] {
	$w.cont.text insert end "$c\n"
    }
}
proc AliasEdit2 {w l id done} {
    global t option
    upvar #0 $id hd

    if $done {
	set content {}
	foreach item [split [$w.cont.text get 0.0 end] ", \t\n"] {
	    if ![regexp {^[ 	]*$} $item] {
		set content "$content,$item"
	    }
	}
	set content [string trimleft $content ,]
	if { ![string length $hd(name)] || ![string length $content] } {
	    Popup $t(need_name_and_content)
	    return
	}
	RatAlias add $hd(name) $hd(fname) $content
	RatAlias save $option(aliases_file)
	# Redraw list
	set top [lindex [$l yview] 0]
	AliasPopulate $l
	$l yview moveto $top
    }

    RecordPos $w aliasEdit
    destroy $w
}
